<!--
SPDX-FileCopyrightText: © 2020 Brian S. Stephan <bss@incorporeal.org>
SPDX-License-Identifier: AGPL-3.0-or-later
-->

{% extends "base.html" %}

{% block header %}
<div class="header">
    <a href="/">{{ config.TITLE_SUFFIX }}</a>
</div>
{% endblock %}

{% block body %}
<div class="content">
    <h1>INTERNAL SERVER ERROR</h1>
    <p>Something bad happened! Please email me at {{ config.CONTACT_EMAIL }} and tell me what happened.</p>
</div>
{% endblock %}
