<!--
SPDX-FileCopyrightText: © 2020 Brian S. Stephan <bss@incorporeal.org>
SPDX-License-Identifier: AGPL-3.0-or-later
-->

{% extends "base.html" %}
{% block site_class %}class="site-wrap site-wrap-double-width"{% endblock %}
