/*
 * SPDX-FileCopyrightText: © 2020 Brian S. Stephan <bss@incorporeal.org>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 */

/* specify almost no styling, just fix some image and nav rendering */
div.header {
    display: flex;
    justify-content: space-between;
}

.img-25 {
    max-width: 25% !important;
}

.img-50 {
    max-width: 50% !important;
}

.img-75 {
    max-width: 75% !important;
}

.img-center {
    display: block;
    margin-left: auto;
    margin-right: auto;
}

.img-left {
    float: left;
    margin-right: 1em;
}

.img-right {
    float: right;
    margin-left: 1em;
}
